package org.pweitz.reactnative.locationswitch;

import android.Manifest;
import android.app.Activity;
import android.content.Context;
import android.content.IntentSender;
import android.content.pm.PackageManager;
import android.location.LocationManager;
import android.os.Build;
import android.support.v4.app.ActivityCompat;
import android.util.Log;

import com.facebook.react.bridge.Callback;
import com.facebook.react.bridge.ReactApplicationContext;
import com.huawei.hmf.tasks.Task;

import com.huawei.hmf.tasks.OnFailureListener;
import com.huawei.hmf.tasks.OnSuccessListener;
import com.huawei.hms.common.ApiException;
import com.huawei.hms.common.ResolvableApiException;
import com.huawei.hms.location.FusedLocationProviderClient;
import com.huawei.hms.location.LocationAvailability;
import com.huawei.hms.location.LocationCallback;
import com.huawei.hms.location.LocationRequest;
import com.huawei.hms.location.LocationResult;
import com.huawei.hms.location.LocationServices;
import com.huawei.hms.location.LocationSettingsRequest;
import com.huawei.hms.location.LocationSettingsResponse;
import com.huawei.hms.location.LocationSettingsResult;
import com.huawei.hms.location.LocationSettingsStatusCodes;
import com.huawei.hms.location.SettingsClient;
import com.huawei.hms.support.api.client.PendingResult;
import com.huawei.hms.support.api.client.ResultCallback;
import com.huawei.hms.support.api.client.Status;

public class LocationSwitch {

    private static final String TAG = "LocationSwitch";
    protected static final int REQUEST_CHECK_SETTINGS = 0x1;

    private static final LocationSwitch instance = new LocationSwitch();

    private int mInterval = 1000;
    private Callback mErrorCallback;
    private Callback mSuccessCallback;
    private int mAccuracy = LocationRequest.PRIORITY_BALANCED_POWER_ACCURACY;

    LocationRequest mLocationRequest;

    private SettingsClient settingsClient;


    private LocationSwitch() {

    }


    public static LocationSwitch getInstance() {
        return instance;
    }


    public void setup(final Callback successCallback, final Callback errorCallback,
                      final int interval, final boolean requestHighAccuracy) {
        mInterval = interval;
        mErrorCallback = errorCallback;
        mSuccessCallback = successCallback;

        if (requestHighAccuracy) {
            mAccuracy = LocationRequest.PRIORITY_HIGH_ACCURACY;
        } else {
            mAccuracy = LocationRequest.PRIORITY_BALANCED_POWER_ACCURACY;
        }
    }


    public void displayLocationSettingsRequest(final Activity activity, final ReactApplicationContext reactContext) {

        LocationRequest locationRequest = LocationRequest.create();
        locationRequest.setPriority(mAccuracy);
        locationRequest.setInterval(mInterval);
        locationRequest.setFastestInterval(mInterval / 2);

        settingsClient = LocationServices.getSettingsClient(activity);
        try {
            LocationSettingsRequest.Builder builder = new LocationSettingsRequest.Builder();
            builder.addLocationRequest(mLocationRequest);
            LocationSettingsRequest locationSettingsRequest = builder.build();
            // check devices settings before request location updates.
            settingsClient.checkLocationSettings(locationSettingsRequest)
                    .addOnSuccessListener(new OnSuccessListener<LocationSettingsResponse>() {
                        @Override
                        public void onSuccess(LocationSettingsResponse locationSettingsResponse) {
                            Log.i(TAG, "check location settings success");
                        }
                    })
                    .addOnFailureListener(new OnFailureListener() {
                        @Override
                        public void onFailure(Exception e) {
                            int statusCode = ((ApiException) e).getStatusCode();
                            switch (statusCode) {
                                case LocationSettingsStatusCodes.RESOLUTION_REQUIRED:
                                    try {
                                        ResolvableApiException rae = (ResolvableApiException) e;
                                        rae.startResolutionForResult(activity, 0);
                                    } catch (IntentSender.SendIntentException sie) {
                                        Log.i(TAG, "PendingIntent unable to execute request.");
                                    }
                                    break;
                                default:
                                    if (ActivityCompat.checkSelfPermission(reactContext,
                                            Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED
                                            && ActivityCompat.checkSelfPermission(reactContext,
                                            Manifest.permission.ACCESS_COARSE_LOCATION) != PackageManager.PERMISSION_GRANTED
                                            && ActivityCompat.checkSelfPermission(reactContext,
                                            "android.permission.ACCESS_BACKGROUND_LOCATION") != PackageManager.PERMISSION_GRANTED) {
                                        String[] strings = {android.Manifest.permission.ACCESS_FINE_LOCATION,
                                                android.Manifest.permission.ACCESS_COARSE_LOCATION,
                                                "android.permission.ACCESS_BACKGROUND_LOCATION"};
                                        ActivityCompat.requestPermissions(activity, strings, 2);
                                    }
                                    break;
                            }
                        }
                    });
        } catch (Exception e) {
            Log.i("Test", "requestLocationUpdatesWithCallback exception:" + e.getMessage());
        }

    }


    public void onActivityResult(int requestCode, int resultCode) {
        switch (requestCode) {
            case REQUEST_CHECK_SETTINGS:
                switch (resultCode) {
                    case Activity.RESULT_OK:
                        callSuccessCallback();
                        break;
                    case Activity.RESULT_CANCELED:
                        callErrorCallback();
                        break;
                }
                break;
        }
    }


    public void isLocationEnabled(final Activity activity, final Callback successCallback,
                                  final Callback errorCallback) {

        LocationManager lm = (LocationManager) activity.getSystemService(Context.LOCATION_SERVICE);
        boolean gps_enabled = false;
        boolean network_enabled = false;

        try {
            gps_enabled = lm.isProviderEnabled(LocationManager.GPS_PROVIDER);
        } catch(Exception ex) {}

        try {
            network_enabled = lm.isProviderEnabled(LocationManager.NETWORK_PROVIDER);
        } catch(Exception ex) {}

        if(gps_enabled || network_enabled) {
            successCallback.invoke();
        } else {
            errorCallback.invoke();
        }
    }


    private void callSuccessCallback() {
        if (null != mSuccessCallback) {
            mSuccessCallback.invoke();
        }
    }


    private void callErrorCallback() {
        if (null != mErrorCallback) {
            mErrorCallback.invoke();
        }
    }

}
